package fucoin.configurations;

import akka.actor.ActorRef;
import akka.actor.ActorSystem;
import akka.actor.Props;
import fucoin.actions.join.ActionTellSupervisor;
import fucoin.supervisor.SuperVisorImpl;
import fucoin.wallet.WalletImpl;

import java.util.ArrayList;
import java.util.List;

/**
 *
 */
public abstract class AbstractConfiguration {

    protected ActorSystem cSystem;
    protected ActorRef superVisor;

    protected final List<ActorRef> activeActors = new ArrayList<>();

    public void setSystem(ActorSystem system) {
        cSystem = system;
    }

    public ActorRef spawnWallet(String name, boolean createGUI) {
        Props props;
        int numOfWallets = activeActors.size();
        if (numOfWallets == 0) {
            props = WalletImpl.props(null, name, createGUI);
        } else {
            props = WalletImpl.props(activeActors.get(numOfWallets - 1), name, createGUI);
        }

        ActorRef actorRef = cSystem.actorOf(props, name);

        activeActors.add(actorRef);

        if (numOfWallets == 0) {
            //actorRef.tell(new ActionJoinAnswer(cSuperVisorActor), cSuperVisorActor);
            actorRef.tell(new ActionTellSupervisor(superVisor), superVisor);
        }

        return actorRef;
    }

    public void spawnWallets(int n, boolean createGUI) {
        for (int i = 0; i < n; i++) {
            String nameOfTheWallet = "Wallet" + String.valueOf(activeActors.size());
            spawnWallet(nameOfTheWallet, createGUI);
        }
    }

    public ActorRef initSupervisor() {
        superVisor = cSystem.actorOf(SuperVisorImpl.props(), "SuperVisorImpl");

        // Don't ask.
        try {
            Thread.sleep(200);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        return superVisor;
    }

    public abstract void run();

    public abstract String getName();

    @Override
    public String toString() {
        return getName();
    }
}
